summaryrefslogtreecommitdiffstats
path: root/Objects/exceptions.c
diff options
context:
space:
mode:
authorIrit Katriel <1055913+iritkatriel@users.noreply.github.com>2022-11-08 09:32:20 (GMT)
committerGitHub <noreply@github.com>2022-11-08 09:32:20 (GMT)
commitc43714fbcdb9bb0927872d6ebf5697edd2e2a1e9 (patch)
treeaa3295656ec6cb571145c0eb89cee906eea94805 /Objects/exceptions.c
parenta751bf565c731c87f2e02678ee080f04c86d7eee (diff)
downloadcpython-c43714fbcdb9bb0927872d6ebf5697edd2e2a1e9.zip
cpython-c43714fbcdb9bb0927872d6ebf5697edd2e2a1e9.tar.gz
cpython-c43714fbcdb9bb0927872d6ebf5697edd2e2a1e9.tar.bz2
gh-99181: fix except* on unhashable exceptions (GH-99192)
Diffstat (limited to 'Objects/exceptions.c')
-rw-r--r--Objects/exceptions.c43
1 files changed, 26 insertions, 17 deletions
diff --git a/Objects/exceptions.c b/Objects/exceptions.c
index 4b4f31a..fd63095 100644
--- a/Objects/exceptions.c
+++ b/Objects/exceptions.c
@@ -962,11 +962,11 @@ typedef enum {
EXCEPTION_GROUP_MATCH_BY_TYPE = 0,
/* A PyFunction returning True for matching exceptions */
EXCEPTION_GROUP_MATCH_BY_PREDICATE = 1,
/* A set of leaf exceptions to include in the result.
* This matcher type is used internally by the interpreter
* to construct reraised exceptions.
*/
- EXCEPTION_GROUP_MATCH_INSTANCES = 2
+ EXCEPTION_GROUP_MATCH_INSTANCE_IDS = 2
} _exceptiongroup_split_matcher_type;
static int
@@ -1024,10 +1024,16 @@ exceptiongroup_split_check_match(PyObject *exc,
Py_DECREF(exc_matches);
return is_true;
}
- case EXCEPTION_GROUP_MATCH_INSTANCES: {
+ case EXCEPTION_GROUP_MATCH_INSTANCE_IDS: {
assert(PySet_Check(matcher_value));
if (!_PyBaseExceptionGroup_Check(exc)) {
- return PySet_Contains(matcher_value, exc);
+ PyObject *exc_id = PyLong_FromVoidPtr(exc);
+ if (exc_id == NULL) {
+ return -1;
+ }
+ int res = PySet_Contains(matcher_value, exc_id);
+ Py_DECREF(exc_id);
+ return res;
}
return 0;
}
@@ -1212,32 +1218,35 @@ BaseExceptionGroup_subgroup(PyObject *self, PyObject *args)
}
static int
-collect_exception_group_leaves(PyObject *exc, PyObject *leaves)
+collect_exception_group_leaf_ids(PyObject *exc, PyObject *leaf_ids)
{
if (Py_IsNone(exc)) {
return 0;
}
assert(PyExceptionInstance_Check(exc));
- assert(PySet_Check(leaves));
+ assert(PySet_Check(leaf_ids));
- /* Add all leaf exceptions in exc to the leaves set */
+ /* Add IDs of all leaf exceptions in exc to the leaf_ids set */
if (!_PyBaseExceptionGroup_Check(exc)) {
- if (PySet_Add(leaves, exc) < 0) {
+ PyObject *exc_id = PyLong_FromVoidPtr(exc);
+ if (exc_id == NULL) {
return -1;
}
- return 0;
+ int res = PySet_Add(leaf_ids, exc_id);
+ Py_DECREF(exc_id);
+ return res;
}
PyBaseExceptionGroupObject *eg = _PyBaseExceptionGroupObject_cast(exc);
Py_ssize_t num_excs = PyTuple_GET_SIZE(eg->excs);
/* recursive calls */
for (Py_ssize_t i = 0; i < num_excs; i++) {
PyObject *e = PyTuple_GET_ITEM(eg->excs, i);
- if (_Py_EnterRecursiveCall(" in collect_exception_group_leaves")) {
+ if (_Py_EnterRecursiveCall(" in collect_exception_group_leaf_ids")) {
return -1;
}
- int res = collect_exception_group_leaves(e, leaves);
+ int res = collect_exception_group_leaf_ids(e, leaf_ids);
_Py_LeaveRecursiveCall();
if (res < 0) {
return -1;
@@ -1258,8 +1267,8 @@ exception_group_projection(PyObject *eg, PyObject *keep)
assert(_PyBaseExceptionGroup_Check(eg));
assert(PyList_CheckExact(keep));
- PyObject *leaves = PySet_New(NULL);
- if (!leaves) {
+ PyObject *leaf_ids = PySet_New(NULL);
+ if (!leaf_ids) {
return NULL;
}
@@ -1268,8 +1277,8 @@ exception_group_projection(PyObject *eg, PyObject *keep)
PyObject *e = PyList_GET_ITEM(keep, i);
assert(e != NULL);
assert(_PyBaseExceptionGroup_Check(e));
- if (collect_exception_group_leaves(e, leaves) < 0) {
- Py_DECREF(leaves);
+ if (collect_exception_group_leaf_ids(e, leaf_ids) < 0) {
+ Py_DECREF(leaf_ids);
return NULL;
}
}
@@ -1277,9 +1286,9 @@ exception_group_projection(PyObject *eg, PyObject *keep)
_exceptiongroup_split_result split_result;
bool construct_rest = false;
int err = exceptiongroup_split_recursive(
- eg, EXCEPTION_GROUP_MATCH_INSTANCES, leaves,
+ eg, EXCEPTION_GROUP_MATCH_INSTANCE_IDS, leaf_ids,
construct_rest, &split_result);
- Py_DECREF(leaves);
+ Py_DECREF(leaf_ids);
if (err < 0) {
return NULL;
}
@@ -962,11 +962,11 @@ typedef enum {
/* A PyFunction returning True for matching exceptions */
EXCEPTION_GROUP_MATCH_BY_PREDICATE = 1,
- /* A set of leaf exceptions to include in the result.
+ /* A set of the IDs of leaf exceptions to include in the result.
* This matcher type is used internally by the interpreter
* to construct reraised exceptions.